/** Syntax Highlighting Test File for C
 *  \brief Doxygen tag highlighting
 */

#include"stdlib.h"

int main()
{
    int val1 = 10;
    int val2 = val1 * 2;
    char mychar = 'A';

    for(;;)
    {
        val2 = val1 - 1;
        if(val <= 0) break;
    }

    printf("Hello World, %d + %d = %d", val1, val2, val1 + val2);
    printf("An unclosed string and unmatched brace
}/*End Main*/

void *test(void) {;}

/*EOF*/
