###############################################################################
# Name: __init__.py                                                           #
# Purpose: Initializes the autocomp package                                   #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
# Licence: wxWindows Licence                                                  #
###############################################################################

"""Autocomp sub package
@note: this may be removed when the autocomp system becomes part of the plugin
       system

"""
__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: __init__.py 62467 2009-10-22 02:10:51Z CJP $"
__revision__ = "$Revision: 62467 $"

#-----------------------------------------------------------------------------#
# Setup Public Namespace

from autocomp import *
from completer import *
