// Syntax Highlighting Test File for haXe
// Comments are like this
/* Mulitline comments are like
 * this block of text
 */

// Ints / Floats
0;
-1234;
0xFF00;
123.4;

// Strings
"Hello World"; 
"Unclosed String;

// Constants
true;
false;
null;

// Class Def
package my.pack;

class MyClass {
    var id : Int;
    static var name : String = "MyString";
    function foo() : Void {
    }
    static function bar( s : String, v : Bool ) : Void {
    }
}
