-- Syntax Highlighting test file for Ada
-- Some Comments about this file
pragma Name (Parameter_List);
with Ada.Text_IO; 

procedure Hello is
begin
   Ada.Text_IO.Put_Line("Hello, world!");
   Ada.Text_IO.Put_Line("Unclosed string);
end Hello;

My_Number := 1;
_Ilegal_Identifier := 'a';
